@@ -2,7 +2,7 @@
 from __future__ import absolute_import, division, print_function, unicode_literals
 
 __author__ = """Michal Odehnal <modehnal@redhat.com>"""
-__version__ = "3.0"
+__version__ = "3.5"
 __copyright__ = "Copyright © 2018-2020 Red Hat, Inc."
 __license__ = "GPL"
 __all__ = ("application",
@@ -13,7 +13,8 @@ __all__ = ("application",
            "logger",
            "online_accounts",
            "sandbox",
-           "utility")
+           "utility",
+           "icons")
 
 # For compatibility reasons as utils source code was in this file.
 from qecore.utility import *
