@@ -71,6 +71,7 @@ class Application:
         self.desktop_file_path = desktop_file_path
         self.application_list = []
         self.app_process_name = app_process_name if app_process_name else component
+        self.icon = None
         self.get_desktop_file_data()
 
         if 'Flatpak' not in f'{type(self)}':
@@ -100,8 +101,8 @@ class Application:
             else:
                 desktop_run = run(" ".join((
                     f"rpm -qlf $(which {self.component}) |",
-                    f"grep /usr/share/applications/ |",
-                    f"grep .desktop |",
+                    "grep /usr/share/applications/ |",
+                    "grep .desktop |",
                     f"grep '{self.desktop_file_name}'"
                 )), verbose=True)
                 if desktop_run[1] != 0:
@@ -109,11 +110,27 @@ class Application:
 
                 desktop_file = desktop_run[0].strip("\n")
 
+            desktop_files_list = desktop_file.split("\n")
+            if len(desktop_files_list) != 1:
+                assert False, "\n".join((
+                    f"More than one .desktop file found: \n{desktop_file}\n",
+                    "Specify one when initializing your application in environment.py, options are:",
+                    "1) Specify your <name>.desktop file name 'desktop_file_name=\"<name>\"'.",
+                    "2) Provide full path to your .desktop file 'desktop_file_path=\"/path/to/desktop/file\"'.",
+                    " ... = context.sandbox.get_application(..., desktop_file_name=\"\", desktop_file_path=\"\")\n"
+                ))
+
             desktop_file_config = configparser.RawConfigParser()
-            desktop_file_config.read(desktop_file)
+            list_of_successfully_parsed_files = desktop_file_config.read(desktop_file)
+
+            if list_of_successfully_parsed_files == []:
+                assert False, "".join((
+                    f"Failed attepmt to parse the .desktop file: '{desktop_file}'."
+                ))
 
             self.name = desktop_file_config.get("Desktop Entry", "name")
             self.exec = desktop_file_config.get("Desktop Entry", "exec").split(" ")[0]
+            self.icon = desktop_file_config.get("Desktop Entry", "icon", fallback=None)
 
 
     def start_via_command(self, command=None, **kwargs):
@@ -202,7 +219,7 @@ class Application:
         if not self.desktop_file_exists:
             raise Exception("".join((
                 f"Testing target '{self.a11y_app_name}' does not have desktop file. ",
-                f"Indication of user failure!"
+                "Indication of user failure!"
             )))
 
         kill_override = kill if kill else self.kill
@@ -247,7 +264,7 @@ class Application:
         if not self.is_running():
             raise Exception("".join((
                 f"Application '{self.a11y_app_name}' is no longer running. ",
-                f"Indication of test failure!"
+                "Indication of test failure!"
             )))
 
         keyCombo(self.exit_shortcut)
@@ -288,7 +305,8 @@ class Application:
 
     def is_running(self):
         """
-        Get accessibility root of appllication.
+        Get running state of application.
+        This state is retrieved by checking if application is present in the accessibility tree.
 
         :return: Return state of application. Running or not.
         :rtype: bool
