@@ -96,30 +96,33 @@ class GetNode:
         retry = kwargs.get("retry", False)
         expect_positive = kwargs.get("expect_positive", False)
 
-        a11y_roots = {x.name: x for x in root.applications()}
+        # Root accessibility object's name is given.
+        if a11y_root_name is not None:
+            # Try to find root application name in context.
+            if hasattr(context, a11y_root_name):
+                self.root = getattr(context, a11y_root_name).instance
+            # Try to find root application name in accessibility tree.
+            else:
+                self.root = root.application(a11y_root_name)
 
-        if a11y_root_name in a11y_roots:
-            self.root = a11y_roots[a11y_root_name]
-        elif a11y_root_name is not None and hasattr(context, a11y_root_name):
-            self.root = getattr(context, a11y_root_name).instance
+        # Root accessibility object's name is not given.
         else:
-            self.root = None
-
-        if self.root is None:
+            # Only option for not having the name is to have default application set.
             try:
                 self.root = context.sandbox.default_application.instance
             except AttributeError:
                 traceback.print_exc(file=sys.stdout)
                 assert False, "".join((
                     "\nYou need to define a default application ",
-                    f"if you are using steps without root."
+                    "if you are using steps without root."
                 ))
 
+        # Condition to do any action on any node, is that the application is running.
         application_list = [x.name for x in root.applications()]
         if not self.root.name in application_list:
-            assert False, " ".join((
-                f"You are trying to do action in application: '{a11y_root_name}'",
-                f"which is not detected running.\n",
+            assert False, "".join((
+                f"You are trying to do action in application: '{self.root.name}' ",
+                "which is not detected running.\n",
                 f"Detected applications are: '{application_list}'"
             ))
 
@@ -133,7 +136,7 @@ class GetNode:
             self.m_btn = mouse_map[str(m_btn)]
         except KeyError:
             traceback.print_exc(file=sys.stdout)
-            assert False, f"\nUnknown mouse button! Check your feature file!\n"
+            assert False, "\nUnknown mouse button! Check your feature file!\n"
 
         self.name = ("".join(name) if name != "Empty" else "") \
             if name != "None" else None
@@ -167,7 +170,7 @@ class GetNode:
             found_node = self.root.findChild(lambda x: \
                 ((self.name is None) or self.name in repr(x.name)) and \
                 ((self.role_name is None) or self.role_name == x.roleName) and \
-                ((self.description is None) or self.description in x.description) and \
+                ((self.description is None) or self.description == x.description) and \
                 ((self.attr is None) or getattr(x, self.attr)) and \
                 Tuple(x.position) >= self.position_low and \
                 Tuple(x.position) <= self.position_high and \
