@@ -3,8 +3,10 @@ import os
 import re
 import base64
 import traceback
+import signal
 from time import sleep
 from subprocess import Popen
+from mimetypes import MimeTypes
 from dogtail.tree import root
 from dogtail.utils import config
 from dogtail.rawinput import keyCombo
@@ -12,6 +14,8 @@ from qecore.utility import run
 from qecore.logger import QELogger
 from qecore.application import Application
 from qecore.flatpak import Flatpak
+from qecore.icons import qecore_icons, QECoreIcon
+
 
 log = QELogger()
 
@@ -66,9 +70,15 @@ class TestSandbox:
 
         self.production = True
 
+        self.timeout_handling = True
+
         self.after_scenario_hooks = []
         self.reverse_after_scenario_hooks = False
 
+        self.change_title = True
+        self.session_icon_to_title = True
+        self.default_application_icon_to_title = False
+
         self.applications = []
         self.default_application = None
         self.retrieve_session_data()
@@ -108,6 +118,12 @@ class TestSandbox:
 
         self.set_up_embedding(context)
 
+        if self.change_title:
+            self.set_title(context)
+
+        if self.timeout_handling:
+            self.set_timeout_handling()
+
         if self.record_video and self.production:
             self.start_recording()
 
@@ -136,13 +152,7 @@ class TestSandbox:
             self.capture_image()
 
         if self.background_image_revert:
-            run(" ".join((
-                "gsettings",
-                "set",
-                "org.gnome.desktop.background",
-                "picture-uri",
-                self.background_image_location
-            )))
+            self.revert_background_image()
 
         if self.record_video:
             self.stop_recording()
@@ -164,25 +174,19 @@ class TestSandbox:
         if self.attach_faf and (self.failed_test or self.attach_faf_on_pass):
             self.attach_abrt_link_to_report(context)
 
-        hook_error = False
+        self.after_scenario_hooks_process(context)
 
-        if self.reverse_after_scenario_hooks:
-            self.after_scenario_hooks.reverse()
 
-        for callback, args, kwargs in self.after_scenario_hooks:
-            try:
-                callback(*args, **kwargs)
-            except Exception as error:
-                hook_error = True
-                error_trace = traceback.format_exc()
-                context.embed("text/plain",
-                              f"Hook Error: {error}\n{error_trace}",
-                              caption="Hook Error")
+    def gracefull_exit(self, signum, frame):
+        """
+        If killed externally, run user defined hooks not to break tests that will be executed next.
 
-        self.after_scenario_hooks = []
+        .. note::
 
-        if hook_error:
-            raise Exception("Exception during after_scenario hook")
+            Do **NOT** call this by yourself. This method is called when killed externally (timeout).
+        """
+
+        assert False, f"Timeout: received signal: '{signum}'"
 
 
     def start_recording(self):
@@ -201,7 +205,7 @@ class TestSandbox:
         self.set_max_video_length_to(600)
 
 
-        leftover_recording_processes = run(f"pgrep -fla qecore_start_recording").strip("\n")
+        leftover_recording_processes = run("pgrep -fla qecore_start_recording").strip("\n")
         if leftover_recording_processes:
             leftover_recording_process_list = leftover_recording_processes.split("\n")
             for process in leftover_recording_process_list:
@@ -406,7 +410,6 @@ class TestSandbox:
         .. note::
 
             Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
-
         """
 
         def embed_data(mime_type, data, caption):
@@ -418,7 +421,14 @@ class TestSandbox:
                     formatter.embedding(mime_type=mime_type, data=non_empty_data, caption=caption)
                     return
 
+        def set_title(title, append=False, tag="span", **kwargs):
+            for formatter in context._runner.formatters:
+                if "html" in formatter.name and getattr(formatter, "set_title", None) is not None:
+                    formatter.set_title(title=title, append=append, tag=tag, **kwargs)
+                    return
+
         context.embed = embed_data
+        context.set_title = set_title
 
 
     def add_after_scenario_hook(self, callback, *args, **kwargs):
@@ -445,6 +455,9 @@ class TestSandbox:
 
             sandbox.add_after_scenario_hook(something)
 
+            # generic function call
+            sandbox.add_after_scenario_hook(function_name, arg1, arg2, kwarg1=val1, ...)
+
             # call command
             sandbox.add_after_scenario_hook(subprocess.call, "command to be called", shell=True)
 
@@ -453,12 +466,25 @@ class TestSandbox:
 
             # embed data computed later (read log file)
             sandbox.add_after_scenario_hook(lambda context: context.embed("text/plain", open(log_file).read(), caption="LOG"), context)
-
         """
 
         self.after_scenario_hooks += [(callback, args, kwargs)]
 
 
+    def set_timeout_handling(self):
+        """
+        Set up signal handling.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
+        """
+
+        signal.signal(signal.SIGTERM, self.gracefull_exit)
+        run("touch /tmp/qecore_timeout_handler")
+
+
+
     def set_animations(self):
         """
         Set animations via gsettings command.
@@ -558,8 +584,8 @@ class TestSandbox:
             log.info(f"set_max_video_length_to(self, number={number})")
 
         run(" ".join((
-            f"gsettings set",
-            f"org.gnome.settings-daemon.plugins.media-keys",
+            "gsettings set",
+            "org.gnome.settings-daemon.plugins.media-keys",
             f"max-screencast-length {number}"
         )))
 
@@ -609,7 +635,6 @@ class TestSandbox:
         .. note::
 
             This method is called by :func:`before_scenario`. You can overwrite the setting.
-
         """
 
         if self.logging:
@@ -649,7 +674,7 @@ class TestSandbox:
             log.info("copy_data_folder(self)")
 
         if os.path.isdir("data/"):
-            run(f"rsync -r data/ /tmp/")
+            run("rsync -r data/ /tmp/")
 
 
     def detect_keyring(self):
@@ -680,7 +705,7 @@ class TestSandbox:
             self.shell.child("Continue").click()
             self.shell.child("Continue").click()
 
-            run(f"touch /tmp/keyring_set")
+            run("touch /tmp/keyring_set")
 
         run("killall qecore_start_keyring")
 
@@ -719,6 +744,74 @@ class TestSandbox:
         if os.path.isfile("/tmp/headless_enable_fatal_critical"):
             os.environ["G_DEBUG"] = "fatal-criticals"
 
+        # Fatal_wanings has bigger priority than criticals.
+        # Should both options be set in Jenkins the warning will overwrite the variable.
+        if os.path.isfile("/tmp/headless_enable_fatal_warnings"):
+            os.environ["G_DEBUG"] = "fatal-warnings"
+
+
+    def set_title(self, context):
+        """
+        Append component name and session type to HTML title.
+
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`before_scenario`.
+        """
+
+        context.set_title("", tag="br", append=True)
+
+        if self.default_application_icon_to_title:
+            icon = self.get_default_application_icon()
+            if icon is not None:
+                context.set_title("", append=True, tag="img",
+                                  alt=self.session_type[1],
+                                  src=icon.to_src(),
+                                  style="height:1.8rem; vertical-align:text-bottom;")
+
+            context.set_title(f"{self.component} - ", append=True, tag="small")
+
+        if self.session_icon_to_title:
+            context.set_title("", append=True, tag="img",
+                              alt=self.session_type[1],
+                              src=qecore_icons[self.session_type].to_src(),
+                              style="height:1.8rem; vertical-align:text-bottom;")
+
+            context.set_title(self.session_type[1:], append=True, tag="small",
+                              style="margin-left:-0.4em;")
+
+        self.change_title = False
+
+
+    def get_default_application_icon(self):
+        """
+        Get icon for default application.
+
+        :return: icon or None
+        :rtype: <icons.QECoreIcon>
+        """
+
+        # Importing here because of sphinx documentation generating issues.
+        import gi
+        gi.require_version('Gtk', '3.0')
+        from gi.repository import Gtk
+
+        if self.default_application and self.default_application.icon:
+            icon_theme = Gtk.IconTheme.get_default()
+            icon = icon_theme.lookup_icon(self.default_application.icon, 48, 0)
+            if icon:
+                icon_path = icon.get_filename()
+                if icon_path:
+                    mime = MimeTypes()
+                    mime_type = mime.guess_type(icon_path)[0]
+                    data_base64 = base64.b64encode(open(icon_path, "rb").read())
+                    data_encoded = data_base64.decode("utf-8").replace("\n", "")
+                    return QECoreIcon(mime_type, "base64", data_encoded)
+        return None
+
 
     def attach_screenshot_to_report(self, context):
         """
@@ -843,9 +936,9 @@ class TestSandbox:
             return
 
         journal_run = run(" ".join((
-            f"sudo journalctl --all",
+            "sudo journalctl --all",
             f"--output=short-precise {self.logging_cursor}",
-            f"> /tmp/journalctl_short.log"
+            "> /tmp/journalctl_short.log"
         )), verbose=True)
 
         if journal_run[1] != 0:
@@ -903,6 +996,56 @@ class TestSandbox:
             context.embed("link", faf_reports, caption="FAF reports")
 
 
+    def after_scenario_hooks_process(self, context):
+        """
+        Process attached after_scenario_hooks.
+
+        :type context: <behave.runner.Context>
+        :param context: Passed object.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
+        """
+
+        hook_error = False
+
+        if self.reverse_after_scenario_hooks:
+            self.after_scenario_hooks.reverse()
+
+        for callback, args, kwargs in self.after_scenario_hooks:
+            try:
+                callback(*args, **kwargs)
+            except Exception as error:
+                hook_error = True
+                error_trace = traceback.format_exc()
+                context.embed("text/plain",
+                              f"Hook Error: {error}\n{error_trace}",
+                              caption="Hook Error")
+
+        self.after_scenario_hooks = []
+
+        assert not hook_error, "Exception during after_scenario hook"
+
+
+    def revert_background_image(self):
+        """
+        Revert background image to the before-test state.
+
+        .. note::
+
+            Do **NOT** call this by yourself. This method is called by :func:`after_scenario`.
+        """
+
+        run(" ".join((
+            "gsettings",
+            "set",
+            "org.gnome.desktop.background",
+            "picture-uri",
+            self.background_image_location
+        )))
+
+
     def wait_for_video_encoding(self, file_name):
         """
         Wait until the video is fully encoded.
@@ -985,7 +1128,7 @@ class TestSandbox:
         self.background_image_revert = background_image_revert
 
         if self.background_image_revert:
-            self.background_image_location = run(f"gsettings get org.gnome.desktop.background picture-uri").strip("\n")
+            self.background_image_location = run("gsettings get org.gnome.desktop.background picture-uri").strip("\n")
 
         if background_image:
             if "file://" in background_image:
@@ -1010,8 +1153,8 @@ class TestSandbox:
         else:
             log.info(" ".join((
                 f"Color '{color}' is not defined.",
-                f"You can define one yourself and submit merge request or"
-                f"email modehnal@redhat.com for support."
+                "You can define one yourself and submit merge request or"
+                "email modehnal@redhat.com for support."
             )))
 
 
@@ -1049,7 +1192,7 @@ class TestSandbox:
             )))
         else:
             assert False, "".join((
-                f"Unknown option, only defined ones are 'show' or 'hide'.",
+                "Unknown option, only defined ones are 'show' or 'hide'.",
                 f"You tried to use: '{action}'"
             ))
 
